#include <motelib/system.h>
#include <motelib/timer.h>
#include <motelib/sensor.h>
#include <motelib/actor.h>
#include <motelib/radio.h>
#include <motelib/uart.h>

#include "messages.h"
#include "sharedvar.h"

#define SENSOR_ACC_X SENSOR_A1
#define SENSOR_ACC_Y SENSOR_A3
#define SENSOR_JOYSTICK_X SENSOR_A2
#define SENSOR_JOYSTICK_Y SENSOR_A4

#define SW_LIGHT_ENABLE 	ACTOR_14
#define SW_MOTION_CONTROL 	ACTOR_15
#define SW_STATION_UP 		ACTOR_16
#define SW_STATION_DOWN 	ACTOR_17

#define STATION_7SEG_1 ACTOR_2
#define STATION_7SEG_2 ACTOR_3
#define STATION_7SEG_3 ACTOR_4
#define STATION_7SEG_4 ACTOR_5

#define STATION_INDICATOR_1 ACTOR_11
#define STATION_INDICATOR_2 ACTOR_12
#define STATION_INDICATOR_3 ACTOR_13

#define JS_X_UPPERBOUND_TRIGGER 1000
#define JS_X_LOWERBOUND_TRIGGER 350
#define JS_Y_UPPERBOUND_TRIGGER 1000
#define JS_Y_LOWERBOUND_TRIGGER 50
#define JS_UPPERBOUND_SHIELD 	800 
#define JS_LOWERBOUND_SHIELD 	600 

#define ACC_UPPERBOUND_TRIGGER 725
#define ACC_LOWERBOUND_TRIGGER 625

Timer acc_t, js_t, sw_t, st1_t, st2_t, st3_t;
//uint16_t acc_x, acc_y;
uint8_t sw_light_state, sw_motion_state, sw_up_state, sw_down_state;
uint8_t js_state;
uint8_t cur_station, cur_color, cur_brightness, cur_light_state;

void readAcc(Timer *t);
void readAccXDone(uint16_t value);
void readAccDone(uint16_t value);
void readJoyStick(Timer *t);
void readJoyStickXDone(uint16_t js_x);
void readJoyStickDone(uint16_t js_y);

void changeStation(uint8_t dir);
void requestStationStatus();
void set7Segment(uint8_t number);

void setLightState();
void changeLightColor(uint8_t dir);
void dimmer(uint8_t dir);
void controlLightDirection(uint8_t direction);

void turnOffStation1(Timer *t);
void turnOffStation2(Timer *t);
void turnOffStation3(Timer *t);

void pirDetectionHandler(uint8_t station);
void getStationStatus(uint8_t enable, uint8_t color, uint8_t bn);

void rxHandler(Address source, MessageType type, void *message, uint8_t len);

void readAcc(Timer *t)
{
	sensorRequestAnalog(SENSOR_ACC_X, readAccXDone);
}

void readAccXDone(uint16_t acc_x)
{
	if(acc_x > ACC_UPPERBOUND_TRIGGER)
	{
		controlLightDirection(LEFT);
	}
	else if(acc_x < ACC_LOWERBOUND_TRIGGER)
	{
		controlLightDirection(RIGHT);
	}

	printf_jennic("\r\n"
					  "ACC_X : %4d\r\n", acc_x);

	sensorRequestAnalog(SENSOR_ACC_Y, readAccDone);
}

void readAccDone(uint16_t acc_y)
{
	if(acc_y > ACC_UPPERBOUND_TRIGGER)
	{
		controlLightDirection(DOWN);
	}
	else if(acc_y < ACC_LOWERBOUND_TRIGGER)
	{
		controlLightDirection(UP);
	}

	printf_jennic("\r\n"
					  "ACC_Y : %4d\r\n", acc_y);
}

void readJoyStick(Timer *t)
{
	sensorRequestAnalog(SENSOR_JOYSTICK_X, readJoyStickXDone);
}

void readJoyStickXDone(uint16_t js_x)
{
	if(cur_light_state == 1)
	{
		if(js_x > JS_X_UPPERBOUND_TRIGGER)
		{
			dimmer(1);
		}
		else if(js_x < JS_X_LOWERBOUND_TRIGGER)
		{
			dimmer(0);
		}
	}

	//printf_jennic("\r\n"
	//				  "JS_X : %4d\r\n",js_x);
	
	sensorRequestAnalog(SENSOR_JOYSTICK_Y, readJoyStickDone);
}

void readJoyStickDone(uint16_t js_y)
{
	if(cur_light_state == 1)
	{
		if(js_state == 0)
		{
			if(js_y > JS_Y_UPPERBOUND_TRIGGER)
			{
				js_state = 1;
				changeLightColor(1);
			}
			else if(js_y < JS_Y_LOWERBOUND_TRIGGER)
			{
				js_state = 2;
				changeLightColor(0);
			}
		}
		else if(js_state == 1)
		{
			if(js_y < (JS_UPPERBOUND_SHIELD))
			{
				js_state = 0;
			}
		}
		else if(js_state == 2)
		{
			if(js_y > (JS_LOWERBOUND_SHIELD))
			{
				js_state = 0;
			}
		}
	}

	//printf_jennic("\r\n"
	//				  "JS_Y : %4d\r\n",js_y);
}

void readSwitch(Timer *t)
{
	if(sw_light_state == 0)
	{
		if(actorGetState(SW_LIGHT_ENABLE) == 0)
		{
			sw_light_state = 1;
			setLightState();
		}
	}
	else if(sw_light_state == 1)
	{
		if(actorGetState(SW_LIGHT_ENABLE) == 1)
			sw_light_state = 0;
	}
	
	if(sw_motion_state == 0)
	{
		if(actorGetState(SW_MOTION_CONTROL) == 0)
		{
			sw_motion_state = 1;
			timerStart(&acc_t, TIMER_PERIODIC, 100, readAcc);
		}
	}
	else if(sw_motion_state == 1)
	{
		if(actorGetState(SW_MOTION_CONTROL) == 1)
		{
			sw_motion_state = 0;
			timerStop(&acc_t);
		}
	}

	if(sw_up_state == 0)
	{
		if(actorGetState(SW_STATION_UP) == 0)
		{
			sw_up_state = 1;
			changeStation(1);
		}
	}
	else if(sw_up_state == 1)
	{
		if(actorGetState(SW_STATION_UP) == 1)
			sw_up_state = 0;
	}

	if(sw_down_state == 0)
	{
		if(actorGetState(SW_STATION_DOWN) == 0)
		{
			sw_down_state = 1;
			changeStation(0);
		}
	}
	else if(sw_down_state == 1)
	{
		if(actorGetState(SW_STATION_DOWN) == 1)
			sw_down_state = 0;
	}

}

void changeStation(uint8_t dir)
{
	if(dir == 0) //decrease current station number
	{
		if(cur_station-1 < 1)
			cur_station = 3;
		else
			cur_station--;
	}
	else if(dir == 1) //increase current station number
	{
		if(cur_station+1 > 3)
			cur_station = 1;
		else
			cur_station++;
	}
	
	set7Segment(cur_station);
}

void requestStationStatus()
{
	radioRequestTx(cur_station,
						TYPE_REQUEST_STATUS,
						NULL,
						0,
						NULL);
}

void set7Segment(uint8_t number)
{
	switch(number)
	{
		case 0:
			actorSetState(STATION_7SEG_1, 0);
			actorSetState(STATION_7SEG_2, 0);
			actorSetState(STATION_7SEG_3, 0);
			actorSetState(STATION_7SEG_4, 0);
			break;
		case 1:
			actorSetState(STATION_7SEG_1, 0);
			actorSetState(STATION_7SEG_2, 0);
			actorSetState(STATION_7SEG_3, 0);
			actorSetState(STATION_7SEG_4, 1);
			break;
		case 2:
			actorSetState(STATION_7SEG_1, 0);
			actorSetState(STATION_7SEG_2, 0);
			actorSetState(STATION_7SEG_3, 1);
			actorSetState(STATION_7SEG_4, 0);
			break;
		case 3:
			actorSetState(STATION_7SEG_1, 0);
			actorSetState(STATION_7SEG_2, 0);
			actorSetState(STATION_7SEG_3, 1);
			actorSetState(STATION_7SEG_4, 1);
			break;
		case 4:
			actorSetState(STATION_7SEG_1, 0);
			actorSetState(STATION_7SEG_2, 1);
			actorSetState(STATION_7SEG_3, 0);
			actorSetState(STATION_7SEG_4, 0);
			break;
		case 5:
			actorSetState(STATION_7SEG_1, 0);
			actorSetState(STATION_7SEG_2, 1);
			actorSetState(STATION_7SEG_3, 0);
			actorSetState(STATION_7SEG_4, 1);
			break;
		case 6:
			actorSetState(STATION_7SEG_1, 0);
			actorSetState(STATION_7SEG_2, 1);
			actorSetState(STATION_7SEG_3, 1);
			actorSetState(STATION_7SEG_4, 0);
			break;
		case 7:
			actorSetState(STATION_7SEG_1, 0);
			actorSetState(STATION_7SEG_2, 1);
			actorSetState(STATION_7SEG_3, 1);
			actorSetState(STATION_7SEG_4, 1);
			break;
		case 8:
			actorSetState(STATION_7SEG_1, 1);
			actorSetState(STATION_7SEG_2, 0);
			actorSetState(STATION_7SEG_3, 0);
			actorSetState(STATION_7SEG_4, 0);
			break;
		case 9:
			actorSetState(STATION_7SEG_1, 1);
			actorSetState(STATION_7SEG_2, 0);
			actorSetState(STATION_7SEG_3, 0);
			actorSetState(STATION_7SEG_4, 1);
			break;
		default:
			break;
	}
}

void setLightState()
{
	//Toggle State
	if(cur_light_state == 1)
		cur_light_state = 0;
	else if(cur_light_state == 0)
		cur_light_state = 1;
	
	radioRequestTx(cur_station, 
						TYPE_LIGHT_ENABLE,
						&cur_light_state,
						sizeof(cur_light_state),
						NULL);
}

void changeLightColor(uint8_t dir)
{
	if(dir == 0) //decrease current color number
	{
		if(cur_color-1 < RED)
			cur_color = WHITE;
		else
			cur_color--;
	}
	else if(dir == 1) //increase current color number
	{
		if(cur_color+1 > WHITE)
			cur_color = RED;
		else
			cur_color++;
	}
	printf_jennic("\r\nColor = %d\r\n", cur_color);

	//send color command to curent station
	radioRequestTx(cur_station,
						TYPE_LIGHT_COLOR,
						&cur_color,
						sizeof(cur_color),
						NULL);
}

void dimmer(uint8_t dir)
{
	if(dir == 0) //decrease brightness
	{
		if(cur_brightness > MIN_BRIGHTNESS)
			cur_brightness--;
	}
	else if(dir == 1) //increase brightness
	{
		if(cur_brightness < MAX_BRIGHTNESS)
			cur_brightness++;
	}

	//send dimmer command to curent station 
	radioRequestTx(cur_station,
						TYPE_DIMMER,
						&cur_brightness,
						sizeof(cur_brightness),
						NULL);

}

void controlLightDirection(uint8_t direction)
{
	radioRequestTx(cur_station,
						TYPE_LIGHT_DIRECTION,
						&direction,
						sizeof(direction),
						NULL);
}

void turnOffStation1(Timer *t)
{
	actorSetState(STATION_INDICATOR_1, 0);
}

void turnOffStation2(Timer *t)
{
	actorSetState(STATION_INDICATOR_2, 0);
}
void turnOffStation3(Timer *t)
{
	actorSetState(STATION_INDICATOR_3, 0);
}

void pirDetectionHandler(uint8_t station)
{
	printf_jennic("\r\nPIR Detection Station : %d\r\n", station);
	switch(station)
	{
		case 1:
			if(timerExpired(&st1_t) == 0)
				timerStop(&st1_t);

			actorSetState(STATION_INDICATOR_1, 1);
			timerStart(&st1_t, TIMER_ONESHOT, 5000, turnOffStation1);
			break;
		case 2:
			if(timerExpired(&st2_t) == 0)
				timerStop(&st2_t);
			
			actorSetState(STATION_INDICATOR_2, 1);
			timerStart(&st2_t, TIMER_ONESHOT, 5000, turnOffStation2);
			break;
		case 3:
			if(timerExpired(&st3_t) == 0)
				timerStop(&st3_t);

			actorSetState(STATION_INDICATOR_3, 1);
			timerStart(&st3_t, TIMER_ONESHOT, 5000, turnOffStation3);
			break;
		default:
			break;
	}
}

void getStationStatus(uint8_t enable, uint8_t color, uint8_t bn)
{
	printf_jennic("\r\nCurrent Status :"
					  "\r\nenable : %d, color : %d, brightness : %d\r\n",
					  enable, color, bn);
	
	cur_light_state = enable;
	cur_color = color;
	cur_brightness = bn;
}

void rxHandler(Address source, MessageType type, void *message, uint8_t len)
{
	printf_jennic("Received pkt from Node ID %d,"
					  "len = %d, type %d\r\n",
					  source, len, type);
	
	if(type == TYPE_PIR_DETECTION)
	{
		pirDetectionHandler(source);
	}
	else if(type == TYPE_STATUS_REPORT)
	{
		getStationStatus(((StationStatus*)message)->enable,
							  ((StationStatus*)message)->color,
							  ((StationStatus*)message)->brightness);
	}
}

void boot()
{
	timerCreate(&acc_t);
	timerCreate(&js_t);
	timerCreate(&sw_t);

	timerCreate(&st1_t);
	timerCreate(&st2_t);
	timerCreate(&st3_t);

	timerStart(&js_t, TIMER_PERIODIC, 100, readJoyStick);
	timerStart(&sw_t, TIMER_PERIODIC, 100, readSwitch);

	uartEnable(true, true);
	radioSetRxHandler(rxHandler);

	sw_light_state = 0;
	sw_motion_state = 0;
	sw_up_state = 0;
	sw_down_state = 0;

	js_state = 0;

	cur_station = 1;
	cur_light_state = 0;
	cur_color = 0;
	cur_brightness = MAX_BRIGHTNESS/2;
	
	set7Segment(cur_station);
	requestStationStatus();
}
